/*
 * File Name      : EsiB1.cs
 *
 * Product        : CICS Transaction Gateway
 *
 * Description    : This sample shows the basic use of the CICS Transaction
 *                  Gateway ESI API. It queries the Gateway for a list of
 *                  available servers, asks the user for a some credentials
 *                  and then verifies the credentials with the server chosen.
 *                  
 * Pre-Requisites : The server must support Password Expiry Management.
 *                                       
 * <copyright 
 * notice="lm-source-program" 
 * pids="CA76" 
 * years="2012" 
 * crc="3406124190" > 
 * Licensed Materials - Property of IBM  
 *  
 * CA76 
 *  
 * (C) Copyright IBM Corp. 2012 All Rights Reserved.  
 *  
 * US Government Users Restricted Rights - Use, duplication or  
 * disclosure restricted by GSA ADP Schedule Contract with  
 * IBM Corp.  
 * </copyright> 
 * 
 * Status: Version 1 Release 0
 *
 *         The following code is sample code created by IBM Corporation. This
 *         sample code is not part of any standard IBM product and is provided
 *         to you solely for the purpose of assisting you in the development of
 *         your applications.  The code is provided 'AS IS', without warranty
 *         or condition of any kind.  IBM shall not be liable for any damages
 *         arising out of your use of the sample code, even if IBM has been
 *         advised of the possibility of such damages.
 */

using System;
using System.Collections.Generic;

// Namespace IBM.CTG contains all the CICS Transaction Gateway types
using IBM.CTG;

namespace IBM.CTG.Samples {

   class EsiB1 {

      // <copyright 
      // notice="lm-string-plus-sample-static" 
      // pids="CA76" 
      // years="2012" 
      // crc="3406124190" > 
      public const string copyright =
         "\n" +
         "Licensed Materials - Property of IBM. " +
         "\n" +
         "CA76" +
         "\n" +
         "(c) Copyright IBM Corp. 2012 All Rights Reserved." +
         "\n" +
         "US Government Users Restricted Rights - Use, duplication or disclosure " +
         "restricted by GSA ADP Schedule with IBM Corp." +
         "\n";
      // </copyright> 

      // Objects used by the class
      private bool   useSsl = false;
      private string gwyHostName = "localhost";
      private int    gwyPort = 2006;

      /// <summary>
      /// Application entry point.
      /// </summary>
      /// <param name="args">Command line arguments passed to the application.</param>
      private static void Main(string[] args) {

         EsiB1 esiApp = new EsiB1();

         // Display program banner
         Console.WriteLine("CICS Transaction Gateway Basic ESI Sample 1");
         Console.WriteLine();

         // Display usage information
         Console.WriteLine("Usage: EsiB1.exe [-ssl] [Gateway host name] [Gateway port number]");
         Console.WriteLine();

         if (esiApp.ParseArguments(args)) {
            try {
               esiApp.Start();
            } catch (Exception e) {
               // Output exception details
               Console.WriteLine(e.ToString());
            }
         }
      }

      /// <summary>
      /// Parses the command line arguments specified when the application was launched.
      /// </summary>
      /// <param name="args">An array of command line arguments.</param>
      /// <returns><see langword="true"/> if the arguments are parsed successfully; otherwise <see langword="false"/>.</returns>
      public bool ParseArguments(string[] args) {

         int usedArgs = 0;

         // Check for -ssl switch
         if ((args.Length > 0) && args[0].Equals("-ssl", StringComparison.InvariantCultureIgnoreCase)) {
            useSsl = true;
            gwyPort = 8050;
            usedArgs++;
         }

         // Parse Gateway host name
         if (args.Length > usedArgs) {
            gwyHostName = args[usedArgs++];
         }

         // Parse Gateway port number
         if (args.Length > usedArgs) {
            if (!int.TryParse(args[usedArgs++], out gwyPort)) {
               Console.WriteLine("ERROR: Invalid port number specified");
               return false;
            }
         }

         // Any remaining arguments are invalid
         if (args.Length > usedArgs) {
            while (usedArgs < args.Length) {
               Console.WriteLine("ERROR: Unexpected argument: {0}", args[usedArgs++]);
            }
            return false;
         }

         Console.WriteLine("The address of the Gateway has been set to {0} port {1}, using {2}",
            gwyHostName, gwyPort, useSsl ? "SSL" : "TCP/IP");
         Console.WriteLine();

         return true;
      }

      /// <summary>
      /// Connects to the CICS Transaction Gateway and flows an ESI request.
      /// </summary>
      public void Start() {
         // Open the Gateway connection with an infinite timeout period
         // The 'using' C# construct ensures the GatewayConnection is closed
         // when it goes out of scope, even if an exception is thrown

         // A System.Net.SocketException will be thrown if the connection
         // attempt fails, and an IBM.CTG.GatewayException will be thrown if
         // an error occurs in the Gateway daemon
         using (GatewayConnection gwyConnection = OpenConnection()) {
            // Connected to the Gateway daemon

            // Get name of CICS server from the user
            string cicsServer = SelectServer(gwyConnection);
            if (cicsServer == null) {
               // No servers defined, or user chose to quit
               return;
            }

            // Get user ID and password or password phrase to verify on server
            Console.WriteLine();
            Console.WriteLine("Enter your user ID for verification:");
            string userid = Console.ReadLine();
            Console.WriteLine();
            Console.WriteLine("Enter password or password phrase for user {0}:", userid);
            string password = Console.ReadLine();

            // Use ESI to check if password or password phrase is valid
            Console.WriteLine();
            Console.WriteLine("Verifying password or password phrase");
            Console.WriteLine();
            EsiVerifyRequest esiVerifyObject = new EsiVerifyRequest(userid,
                                                                    password,
                                                                    cicsServer);

            // Flow throws a System.Net.SocketException if a connection
            // failure occurs, or an IBM.CTG.GatewayException if an error
            // occurs in the Gateway daemon
            gwyConnection.Flow(esiVerifyObject);

            if (esiVerifyObject.EsiReturnCode == EsiReturnCode.EsiNoError) {
               // Flow succeeded
               Console.WriteLine("Password or password phrase verified.");
               Console.Write("  Account last verified: ");
               Console.WriteLine(esiVerifyObject.LastVerified);
               Console.Write("  Account last accessed: ");
               Console.WriteLine(esiVerifyObject.LastAccessed);
               if (esiVerifyObject.PasswordExpiresSet.Value) {
                  Console.Write("  Account expires: ");
                  Console.WriteLine(esiVerifyObject.PasswordExpires);
               } else {
                  Console.WriteLine("  Account is non-expiring");
               }
               Console.Write("  Account password and password phrase invalid count: ");
               Console.WriteLine(esiVerifyObject.InvalidCount);
            } else if (esiVerifyObject.EsiReturnCode == EsiReturnCode.EsiErrPasswordExpired) {
               Console.WriteLine("Password or password phrase expired.");
            } else {
               // Other error return code
               Console.WriteLine("ESI return code: {0} ({1})", esiVerifyObject.ReturnCodeString, esiVerifyObject.ReturnCode);
            }
         }
      }

      /// <summary>
      /// Connects to the CICS Transaction Gateway using TCP/IP or SSL.
      /// </summary>
      /// <returns>The <see cref="GatewayConnection"/> representing the connection to the Gateway daemon.</returns>
      private GatewayConnection OpenConnection() {
         if (useSsl) {
            return new SslGatewayConnection(gwyHostName, gwyPort);
         } else {
            return new GatewayConnection(gwyHostName, gwyPort);
         }
      }

      /// <summary>
      /// Prompts the user to select a server to verify credentials with.
      /// </summary>
      /// <param name="gwyConnection">The connection to the Gateway daemon.</param>
      /// <returns>The name of the CICS server, as defined in the CICS Transaction Gateway.</returns>
      private string SelectServer(GatewayConnection gwyConnection) {

         IList<CicsServer> servers;
         string            chosenServer = null;

         // Retrieve list of CICS servers from the Gateway

         // ListSystems throws a System.Net.SocketException if a connection
         // failure occurs, or an IBM.CTG.GatewayException if an error occurs
         // in the Gateway daemon
         servers = gwyConnection.ListSystems();

         if (servers.Count == 0) {
            // There are no servers defined in the Gateway
            Console.WriteLine("No CICS servers have been defined in the Gateway");
            return null;
         }

         while (chosenServer == null) {

            // Display a list of available CICS servers
            Console.WriteLine("CICS servers defined in the Gateway:");

            for (int i = 0; i < servers.Count; i++) {
               Console.WriteLine("   {0,2}. {1,-8} - {2}", (i + 1), servers[i].Name, servers[i].Description);
            }
            Console.WriteLine();

            // Prompt for user selection
            Console.Write("Choose server to connect to, or Q to quit: ");
            chosenServer = Console.ReadLine();

            // Parse user selection
            if (chosenServer.Equals("Q", StringComparison.InvariantCultureIgnoreCase)) {
               // User chose to quit
               return null;
            }

            int serverNum;
            if (int.TryParse(chosenServer, out serverNum)) {

               // Validate number
               if ((serverNum < 1) || (serverNum > servers.Count)) {
                  Console.WriteLine("Invalid selection");
                  chosenServer = null;
               } else {
                  chosenServer = servers[serverNum - 1].Name;
               }

            } else {
               // Input was not a number
               Console.WriteLine("Invalid selection");
               chosenServer = null;
            }

            Console.WriteLine();
         }

         return chosenServer;
      }
   }
}
