
' File Name      : EciB1.vb
'
' Product        : CICS Transaction Gateway
'
' Description    : This sample shows the basic use of the CICS Transaction
'                  Gateway ECI API. It queries the Gateway for a list of
'                  available servers and then launches a transaction on the
'                  server chosen.
'
' Pre-Requisites : This sample requires the CICS Program EC01 to be installed
'                  on your CICS server for operation. The CICS server must be
'                  set up to return the contents of the COMMAREA as ASCII
'                  text. If the code page of the application is different
'                  from the code page of the server, data conversion must be
'                  performed at the server by making use of CICS-supplied
'                  resource conversion capabilities, such as the DFHCNV macro
'                  definitions.
'
'                  This sample must be compiled to target the x86 processor
'                  architecture, rather than the default of Any CPU. If the
'                  sample is compiled for Any CPU, it may fail to execute on
'                  64 bit platforms.
'
' <copyright 
' notice="lm-source-program" 
' pids="CA76" 
' years="2012" 
' crc="468724311" > 
' Licensed Materials - Property of IBM  
'  
' CA76 
'  
' (C) Copyright IBM Corp. 2012 All Rights Reserved.  
'  
' US Government Users Restricted Rights - Use, duplication or  
' disclosure restricted by GSA ADP Schedule Contract with  
' IBM Corp.  
' </copyright> 
' 
' Status: Version 1 Release 0
'
'         The following code is sample code created by IBM Corporation. This
'         sample code is not part of any standard IBM product and is provided
'         to you solely for the purpose of assisting you in the development of
'         your applications.  The code is provided 'AS IS', without warranty
'         or condition of any kind.  IBM shall not be liable for any damages
'         arising out of your use of the sample code, even if IBM has been
'         advised of the possibility of such damages.

'Enforce strong type checking
Option Strict On
'Require all variables to be declared
Option Explicit On

Imports System
Imports System.Text
Imports System.Collections.Generic
Imports Microsoft.VisualBasic

'Namespace IBM.CTG contains all the CICS Transaction Gateway types
Imports IBM.CTG

Namespace IBM.CTG.Samples

   Public Class EciB1

      ' <copyright 
      ' notice="lm-vb-static" 
      ' pids="CA76" 
      ' years="2012" 
      ' crc="468724311" > 
      Public Const copyright As String = _
      ControlChars.NewLine + _
      "Licensed Materials - Property of IBM " + _
      "CA76" + _
      ControlChars.NewLine + _
      "(c) Copyright IBM Corp. 2012 All Rights Reserved." + _
      "US Government Users Restricted Rights - Use, duplication or " + _
      "disclosure restricted by GSA ADP Schedule Contract with " + _
      "IBM Corp." + _
      ControlChars.NewLine
      ' </copyright> 

      'Name of the CICS program to execute
      Private Const ProgramName As String = "EC01"

      'The size of the COMMAREA to send to the program
      Private Const CommareaLength As Integer = 18

      'Maximum number of flow attempts
      Private Const MaxFlowAttempts As Integer = 3


      'Objects used by the class
      Private useSsl As Boolean
      Private gwyHostName As String = "localhost"
      Private gwyPort As Integer = 2006

      ''' <summary>
      ''' Application entry point.
      ''' </summary>
      ''' <param name="args">Command line arguments passed to the application.</param>
      Public Shared Sub Main(ByVal args As String())
         Dim eciApp As New EciB1()

         'Display program banner
         Console.WriteLine("CICS Transaction Gateway Basic ECI Sample 1")
         Console.WriteLine()

         'Display usage information
         Console.WriteLine("Usage: EciB1.exe [-ssl] [Gateway host name] [Gateway port number]")
         Console.WriteLine()

         If eciApp.ParseArguments(args) Then
            Try
               eciApp.Start()
            Catch e As Exception
               'Output exception details
               Console.WriteLine(e.ToString())
            End Try
         End If
      End Sub

      ''' <summary>
      ''' Parses the command line arguments specified when the application was launched.
      ''' </summary>
      ''' <param name="args">An array of command line arguments.</param>
      ''' <returns><see langword="true"/> if the arguments are parsed successfully; otherwise <see langword="false"/>.</returns>
      Public Function ParseArguments(ByVal args As String()) As Boolean

         Dim usedArgs As Integer = 0

         'Check for -ssl switch
         If (args.Length > 0) AndAlso args(0).Equals("-ssl", StringComparison.InvariantCultureIgnoreCase) Then
            useSsl = True
            gwyPort = 8050
            usedArgs += 1
         End If

         'Parse Gateway host name
         If args.Length > usedArgs Then
            gwyHostName = args(usedArgs)
            usedArgs += 1
         End If

         'Parse Gateway port number
         If args.Length > usedArgs Then
            If Not Integer.TryParse(args(usedArgs), gwyPort) Then
               Console.WriteLine("ERROR: Invalid port number specified")
               Return False
            End If
            usedArgs += 1
         End If

         'Any remaining arguments are invalid
         If args.Length > usedArgs Then
            While usedArgs < args.Length
               Console.WriteLine("ERROR: Unexpected argument: {0}", args(usedArgs))
               usedArgs += 1
            End While
         End If

         Console.WriteLine("The address of the Gateway has been set to {0} port {1}, using {2}", _
            gwyHostName, gwyPort, IIf(useSsl, "SSL", "TCP/IP"))
         Console.WriteLine()

         Return True
      End Function

      ''' <summary>
      ''' Connects to the CICS Transaction Gateway and flows an ECI request.
      ''' </summary>
      ''' <remarks>
      ''' The user is prompted to select a CICS server to execute the EC01 program on.
      ''' The contents of the COMMAREA returned from the request is displayed as ASCII text.
      ''' </remarks>
      Public Sub Start()
         'Open the Gateway connection with an infinite timeout period
         'The 'Using' VB.NET construct ensures the GatewayConnection is closed
         'when it goes out of scope, even if an exception is thrown

         'A System.Net.SocketException will be thrown if the connection
         'attempt fails, and an IBM.CTG.GatewayException will be thrown if
         'an error occurs in the Gateway daemon
         Using gwyConnection As GatewayConnection = OpenConnection()

            'Connected to the Gateway daemon

            'Get name of CICS server from the user
            Dim cicsServer As String = SelectServer(gwyConnection)
            If cicsServer Is Nothing Then
               'No servers defined, or user chose to quit
               Return
            End If

            'Prepare the EciRequest
            Dim eciReq As EciRequest = New EciRequest()
            eciReq.ServerName = cicsServer
            eciReq.Program = ProgramName
            eciReq.ExtendMode = EciExtendMode.EciNoExtend
            eciReq.SetCommareaData(New Byte(CommareaLength - 1) {})

            'Flow the request
            If FlowRequest(gwyConnection, eciReq) Then

               'Display returned COMMAREA
               Dim commarea As Byte() = eciReq.GetCommareaData()

               'Display as hex
               Console.Write("  HEX: ")
               For i As Integer = 0 To commarea.Length - 1
                  Console.Write("{0:x2}", commarea(i))
               Next
               Console.WriteLine()

               'Convert the byte data into a string, treating the data
               'as ASCII encoded characters. This relies on the CICS program
               'returning ASCII text in the COMMAREA
               Dim commareaStr As String = Encoding.ASCII.GetString(commarea)
               Console.WriteLine("ASCII: {0}", commareaStr)

            End If
         End Using
      End Sub

      ''' <summary>
      ''' Connects to the CICS Transaction Gateway using TCP/IP or SSL.
      ''' </summary>
      ''' <returns>The <see cref="GatewayConnection"/> representing the connection to the Gateway daemon.</returns>
      Private Function OpenConnection() As GatewayConnection
         If useSsl Then
            Return New SslGatewayConnection(gwyHostName, gwyPort)
         Else
            Return New GatewayConnection(gwyHostName, gwyPort)
         End If
      End Function

      ''' <summary>
      ''' Flows the ECI request, up to a maximum of MaxFlowAttempts attempts.
      ''' </summary>
      ''' <param name="gwyConnection">The connection to the Gateway daemon.</param>
      ''' <param name="eciReq">The <see cref="EciRequest"/> to flow.</param>
      ''' <returns><see langword="true"/> if the flow succeeds; <see langword="false"/> if MaxFlowAttempts is reached.</returns>
      ''' <remarks>
      ''' If the flow returns a security error, the user is prompted to enter CICS logon credentials.
      ''' </remarks>
      Private Function FlowRequest(ByVal gwyConnection As GatewayConnection, ByVal eciReq As EciRequest) As Boolean

         Dim attempts As Integer = 0

         While attempts < MaxFlowAttempts

            'Flow throws a System.Net.SocketException if a connection
            'failure occurs, or an IBM.CTG.GatewayException if an error
            'occurs in the Gateway daemon
            gwyConnection.Flow(eciReq)

            'Test return code
            Select Case eciReq.EciReturnCode
               Case EciReturnCode.EciNoError
                  'Flow succeeded
                  Return True

               Case EciReturnCode.EciErrSecurityError
                  'Security error, prompt for CICS logon credentials
                  Console.Write("Enter CICS user id : ")
                  eciReq.UserId = Console.ReadLine()
                  Console.Write("Enter CICS password: ")
                  eciReq.Password = Console.ReadLine()

               Case EciReturnCode.EciErrTransactionAbend
                  'Transaction abended
                  Console.WriteLine("The transaction abended: {0}", eciReq.AbendCode)
                  Return False

               Case EciReturnCode.EciErrNoCics
                  'CICS server not found
                  Console.WriteLine("CICS server not found")
                  Return False

               Case Else
                  'Other error return code
                  Console.WriteLine("ECI return code: {0} ({1})", eciReq.EciReturnCode.ToString(), CType(eciReq.EciReturnCode, Integer))
                  Return False

            End Select

            'Increment number of attempts
            attempts += 1
         End While

         'Number of attempts exceeded MaxFlowAttempts
         Return False
      End Function


      ''' <summary>
      ''' Prompts the user to select a server to execute the EC01 program on.
      ''' </summary>
      ''' <param name="gwyConnection">The connection to the Gateway daemon.</param>
      ''' <returns>The name of the CICS server, as defined in the CICS Transaction Gateway.</returns>
      Private Function SelectServer(ByVal gwyConnection As GatewayConnection) As String

         Dim servers As IList(Of CicsServer)
         Dim chosenServer As String = Nothing

         'Retrieve list of CICS servers from the Gateway

         'ListSystems throws a System.Net.SocketException if a connection
         'failure occurs, or an IBM.CTG.GatewayException if an error occurs
         'in the Gateway daemon
         servers = gwyConnection.ListSystems()

         If servers.Count = 0 Then
            'There are no servers defined in the Gateway
            Console.WriteLine("No CICS servers have been defined in the Gateway")
            Return Nothing
         End If

         While chosenServer Is Nothing

            'Display a list of available CICS servers
            Console.WriteLine("CICS servers defined in the Gateway:")

            For i As Integer = 0 To servers.Count - 1
               Console.WriteLine("   {0,2}. {1,-8} - {2}", (i + 1), servers(i).Name, servers(i).Description)
            Next
            Console.WriteLine()

            'Prompt for user selection
            Console.Write("Choose server to connect to, or Q to quit: ")
            chosenServer = Console.ReadLine()

            'Parse user selection
            If chosenServer.Equals("Q", StringComparison.InvariantCultureIgnoreCase) Then
               'User chose to quit
               Return Nothing
            End If

            Dim serverNum As Integer
            If Integer.TryParse(chosenServer, serverNum) Then

               'Validate number
               If (serverNum < 1) OrElse (serverNum > servers.Count) Then
                  Console.WriteLine("Invalid selection")
                  chosenServer = Nothing
               Else
                  chosenServer = servers(serverNum - 1).Name
               End If

            Else
               'Input was not a number
               Console.WriteLine("Invalid selection")
               chosenServer = Nothing
            End If

            Console.WriteLine()
         End While

         Return chosenServer
      End Function

   End Class

End Namespace

