
' File Name      : EsiB1.vb
'
' Product        : CICS Transaction Gateway
'
' Description    : This sample shows the basic use of the CICS Transaction
'                  Gateway ESI API. It queries the Gateway for a list of
'                  available servers, asks the user for a some credentials
'                  and then verifies the credentials with the server chosen.
'
' Pre-Requisites : The server must support Password Expiry Management.
'                                       
' <copyright 
' notice="lm-source-program" 
' pids="CA76" 
' years="2012" 
' crc="2167966874" > 
' Licensed Materials - Property of IBM  
'  
' CA76 
'  
' (C) Copyright IBM Corp. 2012 All Rights Reserved.  
'  
' US Government Users Restricted Rights - Use, duplication or  
' disclosure restricted by GSA ADP Schedule Contract with  
' IBM Corp.  
' </copyright> 
' 
' Status: Version 1 Release 0
'
'         The following code is sample code created by IBM Corporation. This
'         sample code is not part of any standard IBM product and is provided
'         to you solely for the purpose of assisting you in the development of
'         your applications.  The code is provided 'AS IS', without warranty
'         or condition of any kind.  IBM shall not be liable for any damages
'         arising out of your use of the sample code, even if IBM has been
'         advised of the possibility of such damages.

'Enforce strong type checking
Option Strict On
'Require all variables to be declared
Option Explicit On

Imports System
Imports System.Collections.Generic
Imports Microsoft.VisualBasic

'Namespace IBM.CTG contains all the CICS Transaction Gateway types
Imports IBM.CTG

Namespace IBM.CTG.Samples

   Public Class EsiB1

      ' <copyright 
      ' notice="lm-vb-static" 
      ' pids="CA76" 
      ' years="2012" 
      ' crc="2167966874" > 
      Public Const copyright As String = _
      ControlChars.NewLine + _
      "Licensed Materials - Property of IBM " + _
      "CA76" + _
      ControlChars.NewLine + _
      "(c) Copyright IBM Corp. 2012 All Rights Reserved." + _
      "US Government Users Restricted Rights - Use, duplication or " + _
      "disclosure restricted by GSA ADP Schedule Contract with " + _
      "IBM Corp." + _
      ControlChars.NewLine
      ' </copyright> 

      'Objects used by the class
      Private useSsl As Boolean
      Private gwyHostName As String = "localhost"
      Private gwyPort As Integer = 2006

      ''' <summary>
      ''' Application entry point.
      ''' </summary>
      ''' <param name="args">Command line arguments passed to the application.</param>
      Public Shared Sub Main(ByVal args As String())
         Dim esiApp As New EsiB1()

         'Display program banner
         Console.WriteLine("CICS Transaction Gateway Basic ESI Sample 1")
         Console.WriteLine()

         'Display usage information
         Console.WriteLine("Usage: EsiB1.exe [-ssl] [Gateway host name] [Gateway port number]")
         Console.WriteLine()

         If esiApp.ParseArguments(args) Then
            Try
               esiApp.Start()
            Catch e As Exception
               'Output exception details
               Console.WriteLine(e.ToString())
            End Try
         End If
      End Sub

      ''' <summary>
      ''' Parses the command line arguments specified when the application was launched.
      ''' </summary>
      ''' <param name="args">An array of command line arguments.</param>
      ''' <returns><see langword="true"/> if the arguments are parsed successfully; otherwise <see langword="false"/>.</returns>
      Public Function ParseArguments(ByVal args As String()) As Boolean

         Dim usedArgs As Integer = 0

         'Check for -ssl switch
         If (args.Length > 0) AndAlso args(0).Equals("-ssl", StringComparison.InvariantCultureIgnoreCase) Then
            useSsl = True
            gwyPort = 8050
            usedArgs += 1
         End If

         'Parse Gateway host name
         If args.Length > usedArgs Then
            gwyHostName = args(usedArgs)
            usedArgs += 1
         End If

         'Parse Gateway port number
         If args.Length > usedArgs Then
            If Not Integer.TryParse(args(usedArgs), gwyPort) Then
               Console.WriteLine("ERROR: Invalid port number specified")
               Return False
            End If
            usedArgs += 1
         End If

         'Any remaining arguments are invalid
         If args.Length > usedArgs Then
            While usedArgs < args.Length
               Console.WriteLine("ERROR: Unexpected argument: {0}", args(usedArgs))
               usedArgs += 1
            End While
         End If

         Console.WriteLine("The address of the Gateway has been set to {0} port {1}, using {2}", _
            gwyHostName, gwyPort, IIf(useSsl, "SSL", "TCP/IP"))
         Console.WriteLine()

         Return True
      End Function

      ''' <summary>
      ''' Connects to the CICS Transaction Gateway and flows an ESI request.
      ''' </summary>
      Public Sub Start()
         'Open the Gateway connection with an infinite timeout period
         'The 'Using' VB.NET construct ensures the GatewayConnection is closed
         'when it goes out of scope, even if an exception is thrown

         'A System.Net.SocketException will be thrown if the connection
         'attempt fails, and an IBM.CTG.GatewayException will be thrown if
         'an error occurs in the Gateway daemon
         Using gwyConnection As GatewayConnection = OpenConnection()

            'Connected to the Gateway daemon

            'Get name of CICS server from the user
            Dim cicsServer As String = SelectServer(gwyConnection)
            If cicsServer Is Nothing Then
               'No servers defined, or user chose to quit
               Return
            End If

            ' Get user ID and password or password phrase to verify on server
            Console.WriteLine()
            Console.WriteLine("Enter your user ID for verification:")
            Dim userid As String = Console.ReadLine()
            Console.WriteLine()
            Console.WriteLine("Enter password or password phrase for user {0}:", userid)
            Dim password As String = Console.ReadLine()

            ' Use ESI to check if password or password phrase is valid
            Console.WriteLine()
            Console.WriteLine("Verifying password or password phrase")
            Console.WriteLine()

            Dim esiVerifyObject As EsiVerifyRequest = New EsiVerifyRequest(userid, password, cicsServer)

            ' Flow throws a System.Net.SocketException if a connection
            ' failure occurs, or an IBM.CTG.GatewayException if an error
            ' occurs in the Gateway daemon
            gwyConnection.Flow(esiVerifyObject)

            If esiVerifyObject.EsiReturnCode = EsiReturnCode.EsiNoError Then
               ' Flow succeeded
               Console.WriteLine("Password or password phrase verified.")
               Console.Write("  Account last verified: ")
               Console.WriteLine(esiVerifyObject.LastVerified)
               Console.Write("  Account last accessed: ")
               Console.WriteLine(esiVerifyObject.LastAccessed)
               If esiVerifyObject.PasswordExpiresSet.Value Then
                  Console.Write("  Account expires: ")
                  Console.WriteLine(esiVerifyObject.PasswordExpires)
               Else
                  Console.WriteLine("  Account is non-expiring")
               End If
               Console.Write("  Account password and password phrase invalid count: ")
               Console.WriteLine(esiVerifyObject.InvalidCount)
            ElseIf esiVerifyObject.EsiReturnCode = EsiReturnCode.EsiErrPasswordExpired Then
               Console.WriteLine("password or password phrase expired.")
            Else
               ' Other error return code
               Console.WriteLine("ESI return code: {0} ({1})", esiVerifyObject.ReturnCodeString, esiVerifyObject.ReturnCode)
            End If
         End Using
      End Sub

      ''' <summary>
      ''' Connects to the CICS Transaction Gateway using TCP/IP or SSL.
      ''' </summary>
      ''' <returns>The <see cref="GatewayConnection"/> representing the connection to the Gateway daemon.</returns>
      Private Function OpenConnection() As GatewayConnection
         If useSsl Then
            Return New SslGatewayConnection(gwyHostName, gwyPort)
         Else
            Return New GatewayConnection(gwyHostName, gwyPort)
         End If
      End Function

      ''' <summary>
      ''' Prompts the user to select a server to verify credentials with.
      ''' </summary>
      ''' <param name="gwyConnection">The connection to the Gateway daemon.</param>
      ''' <returns>The name of the CICS server, as defined in the CICS Transaction Gateway.</returns>
      Private Function SelectServer(ByVal gwyConnection As GatewayConnection) As String
         Dim servers As IList(Of CicsServer)
         Dim chosenServer As String = Nothing

         'Retrieve list of CICS servers from the Gateway

         'ListSystems throws a System.Net.SocketException if a connection
         'failure occurs, or an IBM.CTG.GatewayException if an error occurs
         'in the Gateway daemon
         servers = gwyConnection.ListSystems()

         If servers.Count = 0 Then
            'There are no servers defined in the Gateway
            Console.WriteLine("No CICS servers have been defined in the Gateway")
            Return Nothing
         End If

         While chosenServer Is Nothing

            'Display a list of available CICS servers
            Console.WriteLine("CICS servers defined in the Gateway:")

            For i As Integer = 0 To servers.Count - 1
               Console.WriteLine("   {0,2}. {1,-8} - {2}", (i + 1), servers(i).Name, servers(i).Description)
            Next
            Console.WriteLine()

            'Prompt for user selection
            Console.Write("Choose server to connect to, or Q to quit: ")
            chosenServer = Console.ReadLine()

            'Parse user selection
            If chosenServer.Equals("Q", StringComparison.InvariantCultureIgnoreCase) Then
               'User chose to quit
               Return Nothing
            End If

            Dim serverNum As Integer
            If Integer.TryParse(chosenServer, serverNum) Then

               'Validate number
               If (serverNum < 1) OrElse (serverNum > servers.Count) Then
                  Console.WriteLine("Invalid selection")
                  chosenServer = Nothing
               Else
                  chosenServer = servers(serverNum - 1).Name
               End If

            Else
               'Input was not a number
               Console.WriteLine("Invalid selection")
               chosenServer = Nothing
            End If

            Console.WriteLine()
         End While

         Return chosenServer
      End Function

   End Class

End Namespace

